----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:48:29 05/05/2010 
-- Design Name: 
-- Module Name:    top_level - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_level is
    Port ( clk_100_in    : in  STD_LOGIC;
	        lvds_in_p    : in  STD_LOGIC;
           lvds_in_n    : in  STD_LOGIC;
           strobe_in_p  : in  STD_LOGIC;
			  strobe_in_n  : in  STD_LOGIC;
			  data_in      : in  std_logic;
			  led          : out std_logic_vector (7 downto 0) --;
--         seg_o	      : out std_logic_vector(6 downto 0);
--	        segdp_o      : out std_logic;
--	        segan_o      : out std_logic_vector(3 downto 0) --;
--           data_in_i    : in std_logic;
--           data_out_o   : out std_logic			  
			  );
end top_level;

architecture Behavioral of top_level is

component dcm_1 
   PORT(
		CLKIN_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKDV_OUT : OUT std_logic;
		CLKIN_IBUFG_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		CLK2X_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
end component;

COMPONENT dcm_2
	PORT(
--		CLKIN_IN : IN std_logic;
--		RST_IN : IN std_logic;          
--		CLK0_OUT : OUT std_logic;
--		LOCKED_OUT : OUT std_logic
		CLKIN_N_IN : IN std_logic;
		CLKIN_P_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKIN_IBUFGDS_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
END COMPONENT;

	

component seven_seg_x4 
  port (
	 in0		: in	std_logic_vector(3 downto 0);
	 in1		: in	std_logic_vector(3 downto 0);
	 in2		: in	std_logic_vector(3 downto 0);
	 in3		: in	std_logic_vector(3 downto 0);
	 dpin0	: in	std_logic;
	 dpin1	: in	std_logic;
	 dpin2	: in	std_logic;
	 dpin3	: in	std_logic;
	 seg_o	: out std_logic_vector(6 downto 0);
	 segdp_o : out std_logic;
	 segan_o : out std_logic_vector(3 downto 0);
	 tick_i  : in	std_logic;
	 clk		: in	std_logic;
	 rst		: in	std_logic
	 );
end component;

constant int_type : string := "DIFF";

signal lvds_in_data    : std_logic;
signal lvds_out_data   : std_logic;
signal data_lfsr       : std_logic_vector (19 downto 0) := x"00000";
signal data_sr_100     : std_logic_vector (19 downto 0);
signal load            : std_logic := '0';
signal load_t          : std_logic;
signal clk_5           : std_logic;
signal clk_100         : std_logic;
signal dcm_locked      : std_logic;
signal lvds_data_in_d  : std_logic;
signal lvds_data_in_d1 : std_logic;
signal data_dsr_100    : std_logic_vector (19 downto 0);
signal strobe_out      : std_logic;
signal strobe_out_t    : std_logic;
signal strobe_in       : std_logic;
signal strobe_in_t     : std_logic;
signal grab            : std_logic;
signal compare         : std_logic_vector (19 downto 0);
signal data_lfsr_rcv   : std_logic_vector (19 downto 0);
signal err_count       : integer ;
signal seg_temp        : std_logic_vector (15 downto 0);
signal tick_i          : std_logic;
signal tick_count      : integer ;
signal grab_t          : std_logic;
signal counter         : integer ;
type state_type is (IDLE,SYNC,SYNC_1,SYNC_2,RUN);
signal state           : state_type;
signal clk_100_rc      : std_logic;
signal dcm_locked_2    : std_logic;
signal word_count       : std_logic_vector (31 downto 0);
signal r_counter      : integer;
constant s_word    : std_logic_vector (19 downto 0) := x"7FFFF";

begin


Inst_dcm_2: dcm_2 PORT MAP(
--		CLKIN_IN => clk_100,
--		RST_IN => not (dcm_locked),
--		CLK0_OUT => clk_100_rc,
--		LOCKED_OUT => dcm_locked_2
		CLKIN_N_IN => strobe_in_n,
		CLKIN_P_IN => strobe_in_p,
		RST_IN => '0',
		CLKIN_IBUFGDS_OUT => open,
		CLK0_OUT => clk_100_rc,
		LOCKED_OUT => dcm_locked_2
	);

lvds_in : IBUFDS port map (
           O  => lvds_in_data,
			  I  => lvds_in_p,
			  IB => lvds_in_n
			  );

differential :
   if int_type = "DIFF" generate
   begin
	process (clk_100_rc, dcm_locked_2)
   begin
      if dcm_locked_2 = '0' then
         data_dsr_100 <= x"00000";
	      lvds_data_in_d <= '0';
	      lvds_data_in_d1 <= '0';
	   elsif rising_edge (clk_100_rc) then
         lvds_data_in_d <= lvds_in_data;
	      lvds_data_in_d1 <= lvds_data_in_d;
	      data_dsr_100 <= data_dsr_100(18 downto 0) & lvds_data_in_d1;
	   end if;
    end process;
end generate;


process (clk_100_rc, dcm_locked_2)
begin
if dcm_locked_2 = '0' then
   data_lfsr_rcv <=  x"00001";
	err_count <= 0;
	counter <= 0;
	state <= IDLE;
	r_counter <= 0;
elsif rising_edge (clk_100_rc) then
   case state is
   when IDLE =>
        if data_dsr_100 = s_word then
           state <= SYNC;
	     end if;
        counter <= 0;
   when SYNC =>
   	  if counter = 339 then
           if data_dsr_100 = s_word then
              state <= SYNC_1;
           else
              state <= IDLE;
           end if;
           counter <= 0;
		  else
		     counter <= counter + 1;
		  end if;
   when SYNC_1 =>
        if counter = 339 then
           if data_dsr_100 = s_word then
              state <= SYNC_2;
              data_lfsr_rcv <= x"00001";
				  counter <= 0;
           else
              state <= IDLE;
           end if;
        else 
		     counter <= counter + 1;
        end if;
   when SYNC_2 =>
	     if counter = 19 then
		     data_lfsr_rcv  <= data_dsr_100(18 downto 0) & (data_dsr_100(2) xnor data_dsr_100(16));
			  state <= RUN;
			  counter <= 0;
		  else
		     counter <= counter + 1;
		  end if;
	when RUN =>
        if r_counter = 19 then
      	  r_counter <= 0;
			  if data_lfsr_rcv /= data_dsr_100 and data_dsr_100 /= s_word then
		        err_count <= err_count + 1;
			     state <= RUN;
		     end if;
			  word_count <= word_count + '1';
			  if counter /= 319 then
			     data_lfsr_rcv  <= data_lfsr_rcv(18 downto 0) & (data_lfsr_rcv(2) xnor data_lfsr_rcv(16));
           end if;
	     else
	        r_counter <= r_counter + 1;
	     end if;
		  if counter = 339 then
		     counter <= 0;
		  else
		     counter <= counter + 1;
		  end if;
   when others =>
	     state <= IDLE;
		  err_count <= 0;
		  data_lfsr_rcv  <= x"00001";
   end case;
   	
end if;
end process;

--- Just to see error count etc. on the chipscope --
led <= seg_temp (15 downto 8) xor seg_temp (7 downto 0) xor (("00" & '0') & conv_std_logic_vector (counter,5)); -- (4 downto 0));
seg_temp <= conv_std_logic_vector (err_count, 16) xor word_count(31 downto 16) xor word_count (15 downto 0);

--seven_seg_disp : seven_seg_x4 
--  port map (
--	 in0		=> seg_temp (3 downto 0), --: in	std_logic_vector(3 downto 0);
--	 in1		=> seg_temp (7 downto 4), --: in	std_logic_vector(3 downto 0);
--	 in2		=> seg_temp (11 downto 8), --: in	std_logic_vector(3 downto 0);
--	 in3		=> seg_temp (15 downto 12), --: in	std_logic_vector(3 downto 0);
--	 dpin0	=> '0', --: in	std_logic;
--	 dpin1	=> '0', --: in	std_logic;
--	 dpin2	=> '0', --: in	std_logic;
--	 dpin3	=> '0', --: in	std_logic;
--	 seg_o	=> seg_o, --: out std_logic_vector(6 downto 0);
--	 segdp_o => segdp_o, --: out std_logic;
--	 segan_o => segan_o, --: out std_logic_vector(3 downto 0);
--	 tick_i  => tick_i, --: in	std_logic;
--	 clk		=> clk_5, --: in	std_logic;
--	 rst		=> not(dcm_locked) --: in	std_logic
--	 );

--process (clk_5,dcm_locked)
--begin
--if dcm_locked = '0' then
--   tick_i <= '0';
--	tick_count <= 0;
--	
--elsif rising_edge (clk_5) then
--   if tick_count = 1000 then
--	   tick_i <= not(tick_i);
--		tick_count <= 0;
--	else 
--	  tick_count <= tick_count + 1;
--	end if;
--end if;
--end process;	


end Behavioral;

