----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:48:29 05/05/2010 
-- Design Name: 
-- Module Name:    top_level - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_level is
    Port ( clk_50_in    : in  STD_LOGIC;
	        lvds_out_p   : out STD_LOGIC;
           lvds_out_n   : out STD_LOGIC;
           strobe_out_p : out STD_LOGIC;
			  strobe_out_n : out STD_LOGIC;
           data_out_o   : out std_logic			  
			  );
end top_level;

architecture Behavioral of top_level is

component dcm_1 
   port ( CLKIN_IN        : in    std_logic; 
          RST_IN          : in    std_logic; 
          CLKDV_OUT       : out   std_logic; 
          CLKIN_IBUFG_OUT : out   std_logic; 
          CLK0_OUT        : out   std_logic; 
          CLK2X_OUT       : out   std_logic; 
          LOCKED_OUT      : out   std_logic);
end component;

COMPONENT dcm_2
	PORT(
		--CLKIN_IN : IN std_logic;
		CLKIN_N_IN : IN std_logic;
		CLKIN_P_IN : IN std_logic;
		RST_IN : IN std_logic;          
		CLKIN_IBUFGDS_OUT : OUT std_logic;
		CLK0_OUT : OUT std_logic;
		LOCKED_OUT : OUT std_logic
		);
END COMPONENT;

	

component seven_seg_x4 
  port (
	 in0		: in	std_logic_vector(3 downto 0);
	 in1		: in	std_logic_vector(3 downto 0);
	 in2		: in	std_logic_vector(3 downto 0);
	 in3		: in	std_logic_vector(3 downto 0);
	 dpin0	: in	std_logic;
	 dpin1	: in	std_logic;
	 dpin2	: in	std_logic;
	 dpin3	: in	std_logic;
	 seg_o	: out std_logic_vector(6 downto 0);
	 segdp_o : out std_logic;
	 segan_o : out std_logic_vector(3 downto 0);
	 tick_i  : in	std_logic;
	 clk		: in	std_logic;
	 rst		: in	std_logic
	 );
end component;

signal lvds_in_data    : std_logic;
signal lvds_out_data   : std_logic;
signal data_lfsr       : std_logic_vector (19 downto 0) := x"00000";
signal data_sr_100     : std_logic_vector (19 downto 0);
signal load            : std_logic := '0';
signal load_t          : std_logic;
signal clk_5           : std_logic;
signal clk_100         : std_logic;
signal dcm_locked      : std_logic;
signal lvds_data_in_d  : std_logic;
signal lvds_data_in_d1 : std_logic;
signal data_dsr_100    : std_logic_vector (19 downto 0);
signal strobe_out      : std_logic;
signal strobe_out_t    : std_logic;
signal strobe_in       : std_logic;
signal strobe_in_t     : std_logic;
signal grab            : std_logic;
signal compare         : std_logic_vector (19 downto 0);
signal data_lfsr_rcv   : std_logic_vector (19 downto 0);
signal err_count       : integer ;
signal seg_temp        : std_logic_vector (15 downto 0);
signal tick_i          : std_logic;
signal tick_count      : integer ;
signal grab_t          : std_logic;
signal counter         : integer range 0 to 20;
signal state           : std_logic;
signal clk_100_rc      : std_logic;
signal dcm_locked_2    : std_logic;
signal insert_count    : std_logic_vector (63 downto 0);
signal data_out		  : std_logic;
constant s_word    : std_logic_vector (19 downto 0) := x"7FFFF";

begin

clocks: dcm_1 
   port map ( CLKIN_IN        => clk_50_in,
              RST_IN          => '0', 
              CLKDV_OUT       => clk_5,
              CLKIN_IBUFG_OUT => open, 
              CLK0_OUT        => open, 
              CLK2X_OUT       => clk_100, 
              LOCKED_OUT      => dcm_locked
				  );


lvds_out : OBUFDS port map (
           I  => lvds_out_data,
			  O  => lvds_out_p,
			  OB => lvds_out_n
			  );

strobe_diff_o : OBUFDS port map (
         I  => clk_100, 
			O  => strobe_out_p,
			OB => strobe_out_n
			);


data_out_buf : OBUF port map (
         O => data_out_o,
			I => data_out
			);

process (clk_5, dcm_locked) 
variable lfsr_val, lfsr_val_2 : std_logic_vector (19 downto 0);
begin
if dcm_locked = '0' then
   load <= '0';
	data_lfsr <=  x"00000";
   lfsr_val := x"00000";
elsif rising_edge (clk_5) then
   if lfsr_val = s_word then
      lfsr_val_2 := x"fffff";
   else
      lfsr_val_2 := lfsr_val;
   end if;
   if insert_count = x"10" then
      data_lfsr 	<= s_word;
      insert_count <= (others => '0');
   else
      lfsr_val := lfsr_val(18 downto 0) & (lfsr_val(2) xnor lfsr_val(16));
		data_lfsr <= lfsr_val_2;
      insert_count <= insert_count + '1';
   end if;
	load <= not(load);
end if;
end process;	

process (clk_100, dcm_locked) 
begin
if dcm_locked = '0' then
   load_t <= '0';
	data_sr_100 <=  x"00000";
elsif rising_edge (clk_100) then
   load_t <= load;
	data_sr_100 <= data_sr_100 (18 downto 0) & data_sr_100 (19);
	if (load_t xor load )= '1' then
	   data_sr_100 <= data_lfsr;
	end if;
	lvds_out_data <= data_sr_100 (19);
	data_out <= data_sr_100 (19);
end if;
end process;

end Behavioral;

